unit pocketsphinx;

{$mode objfpc}{$H+}

interface

uses sysutils;

type
  TpsArg = record
   name: PChar;
   typ: integer;
   def, doc: pchar;
		end;

  PpsArg = ^TpsArg;

  PpsConfig = pointer;
  PpsDecoder = pointer;

  PpsBuffer = ^int16;

const
  PS_lib_name = 'libpocketsphinx.so.3';
  SB_lib_name = 'libsphinxbase.so.3';

  //TpsArgS types
  ARG_REQUIRED = 1;
  ARG_INTEGER = 2;
  ARG_FLOATING = 4;
  ARG_STRING = 8;
  ARG_BOOLEAN = 16;
  REQARG_INTEGER = ARG_INTEGER + ARG_REQUIRED;
  REQARG_FLOATING = ARG_FLOATING + ARG_REQUIRED;
  REQARG_STRING = ARG_STRING + ARG_REQUIRED;
  REQARG_BOOLEAN = ARG_BOOLEAN + ARG_REQUIRED;
  ARG_INT32 = ARG_INTEGER;
  ARG_FLOAT32 = ARG_FLOATING;
  ARG_FLOAT64 = ARG_FLOATING;
  REQARG_INT32 = ARG_INT32 + ARG_REQUIRED;
  REQARG_FLOAT32 = ARG_FLOAT32 + ARG_REQUIRED;
  REQARG_FLOAT64 = ARG_FLOAT64 + ARG_REQUIRED;

var
  psDefaultArgs: array [0..115] of TpsArg = (
  // waveform
  (name : '-logspec'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Write out logspectral files instead of cepstra'),
  (name : '-smoothspec'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Write out cepstral-smoothed logspectral files'),
  (name : '-transform'; typ : ARG_STRING; def : 'legacy'; doc : 'Which type of transform to use to calculate cepstra (legacy, dct, or htk)'),
  (name : '-alpha'; typ : ARG_FLOAT32; def : '0.97'; doc : 'Preemphasis parameter'),
  (name : '-samprate'; typ : ARG_FLOAT32; def : '16000'; doc : 'Sampling rate'),
  (name : '-frate'; typ : ARG_INT32; def : '100'; doc : 'Frame rate'),
  (name : '-wlen'; typ : ARG_FLOAT32; def : '0.025625'; doc : 'Hamming window length'),
  (name : '-nfft'; typ : ARG_INT32; def : '512'; doc : 'Size of FFT'),
  (name : '-nfilt'; typ : ARG_INT32; def : '40'; doc : 'Number of filter banks'),
  (name : '-lowerf'; typ : ARG_FLOAT32; def : '133.33334'; doc : 'Lower edge of filters'),
  (name : '-upperf'; typ : ARG_FLOAT32; def : '6855.4976'; doc : 'Upper edge of filters'),
  (name : '-unit_area'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Normalize mel filters to unit area'),
  (name : '-round_filters'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Round mel filter frequencies to DFT points'),
  (name : '-ncep'; typ : ARG_INT32; def : '13'; doc : 'Number of cep coefficients'),
  (name : '-doublebw'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Use double bandwidth filters (same center freq)'),
  (name : '-lifter'; typ : ARG_INT32; def : '0'; doc : 'Length of sin-curve for liftering, or 0 for no liftering'),
  (name : '-vad_prespeech'; typ : ARG_INT32; def : '20'; doc : 'Num of speech frames to keep before silence to speech'),
  (name : '-vad_startspeech'; typ : ARG_INT32; def : '10'; doc : 'Num of speech frames to trigger vad from silence to speech'),
  (name : '-vad_postspeech'; typ : ARG_INT32; def : '50'; doc : 'Num of silence frames to keep after from speech to silence'),
  (name : '-vad_threshold'; typ : ARG_FLOAT32; def : '2.0'; doc : 'Threshold for decision between noise and silence frames. Log-ratio between signal level and noise level'),
  (name : '-input_endian'; typ : ARG_STRING; def : 'little'; doc : 'Endianness of input data, big or little, ignored if NIST or MS Wav'),
  (name : '-warp_type'; typ : ARG_STRING; def : 'inverse_linear'; doc : 'Warping function type (or shape)'),
  (name : '-warp_params'; typ : ARG_STRING; def : nil; doc : 'Parameters defining the warping function'),
  (name : '-dither'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Add 1/2-bit noise'),
  (name : '-seed'; typ : ARG_INT32; def : '-1'; doc : 'Seed for random number generator; if less than zero, pick our own'),
  (name : '-remove_dc'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Remove DC offset from each frame'),
  (name : '-remove_noise'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Remove noise with spectral subtraction in mel-energies'),
  (name : '-remove_silence'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Enables VAD, removes silence frames from processing'),
  (name : '-verbose'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Show input filenames'),
  //Cepstral
  (name : '-feat'; typ : ARG_STRING; def : '1s_c_d_dd'; doc : 'Feature stream type, depends on the acoustic model'),
  (name : '-ceplen'; typ : ARG_INT32; def : '13'; doc : 'Number of components in the input feature vector'),
  (name : '-cmn'; typ : ARG_STRING; def : 'live'; doc : 'Cepstral mean normalization scheme ("live", "batch", or "none")'),
  (name : '-cmninit'; typ : ARG_STRING; def : '40,3,-1'; doc : 'Initial values (comma-separated) for cepstral mean when "live" is used'),
  (name : '-varnorm'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Variance normalize each utterance (only if CMN == current)'),
  (name : '-agc'; typ : ARG_STRING; def : 'noise'; doc : 'Automatic gain control for c0 ("max", "emax", "noise", or "none")'),
  (name : '-agcthresh'; typ : ARG_FLOAT32; def : '2.0'; doc : 'Initial threshold for automatic gain control'),
  (name : '-lda'; typ : ARG_STRING; def : nil; doc : 'File containing transformation matrix to be applied to features (single-stream features only)'),
  (name : '-ldadim'; typ : ARG_INT32; def : '0'; doc : 'Dimensionality of output of feature transformation (0 to use entire matrix)'),
  (name : '-svspec'; typ : ARG_STRING; def : nil; doc : 'Subvector specification (e.g., 24,0-11/25,12-23/26-38 or 0-12/13-25/26-38)'),
  // ACMOD
  (name : '-hmm'; typ : ARG_STRING; def : '/usr/share/pocketsphinx/def'; doc : 'Directory containing acoustic model files'),
  (name : '-featparams'; typ : ARG_STRING; def : nil; doc : 'File containing feature extraction parameters'),
  (name : '-mdef'; typ : ARG_STRING; def : nil; doc : 'Model definition input file'),
  (name : '-senmgau'; typ : ARG_STRING; def : nil; doc : 'Senone to codebook mapping input file (usually not needed)'),
  (name : '-tmat'; typ : ARG_STRING; def : nil; doc : 'HMM state transition matrix input file'),
  (name : '-tmatfloor'; typ : ARG_FLOAT32; def : '0.0001'; doc : 'HMM state transition probability floor (applied to -tmat file)'),
  (name : '-mean'; typ : ARG_STRING; def : nil; doc : 'Mixture gaussian means input file'),
  (name : '-var'; typ : ARG_STRING; def : nil; doc : 'Mixture gaussian variances input file'),
  (name : '-varfloor'; typ : ARG_FLOAT32; def : '0.0001'; doc : 'Mixture gaussian variance floor (applied to data from -var file)'),
  (name : '-mixw'; typ : ARG_STRING; def : nil; doc : 'Senone mixture weights input file (uncompressed)'),
  (name : '-mixwfloor'; typ : ARG_FLOAT32; def : '0.0000001'; doc : 'Senone mixture weights floor (applied to data from -mixw file)'),
  (name : '-aw'; typ : ARG_INT32; def : '1'; doc : 'Inverse weight applied to acoustic scores'),
  (name : '-sendump'; typ : ARG_STRING; def : nil; doc : 'Senone dump (compressed mixture weights) input file'),
  (name : '-sdmap'; typ : ARG_STRING; def : nil; doc : 'Subspace distribution cluster map file'),
  (name : '-mllr'; typ : ARG_STRING; def : nil; doc : 'MLLR transformation to apply to means and variances'),
  (name : '-mmap'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Use memory-mapped I/O (if possible) for model files'),
  (name : '-ds'; typ : ARG_INT32; def : '1'; doc : 'Frame GMM computation downsampling ratio'),
  (name : '-topn'; typ : ARG_INT32; def : '4'; doc : 'Number of top Gaussians to use in scoring'),
  (name : '-topn_beam'; typ : ARG_STRING; def : '0'; doc : 'Beam width used to determine top-N Gaussians (or a list, per-feature)'),
  (name : '-logbase'; typ : ARG_FLOAT32; def : '1.0001'; doc : 'Base in which all log-likelihoods calculated'),
  // beam
  (name : '-beam'; typ : ARG_FLOAT64; def : '1e-48'; doc : 'Beam width applied to every frame in Viterbi search (smaller values mean wider beam)'),
  (name : '-wbeam'; typ : ARG_FLOAT64; def : '7e-29'; doc : 'Beam width applied to word exits'),
  (name : '-pbeam'; typ : ARG_FLOAT64; def : '1e-48'; doc : 'Beam width applied to phone transitions'),
  (name : '-lpbeam'; typ : ARG_FLOAT64; def : '1e-40'; doc : 'Beam width applied to last phone in words'),
  (name : '-lponlybeam'; typ : ARG_FLOAT64; def : '7e-29'; doc : 'Beam width applied to last phone in single-phone words'),
  (name : '-fwdflatbeam'; typ : ARG_FLOAT64; def : '1e-64'; doc : 'Beam width applied to every frame in second-pass flat search'),
  (name : '-fwdflatwbeam'; typ : ARG_FLOAT64; def : '7e-29'; doc : 'Beam width applied to word exits in second-pass flat search'),
  (name : '-pl_window'; typ : ARG_INT32; def : '5'; doc : 'Phoneme lookahead window size, in frames'),
  (name : '-pl_beam'; typ : ARG_FLOAT64; def : '1e-10'; doc : 'Beam width applied to phone loop search for lookahead'),
  (name : '-pl_pbeam'; typ : ARG_FLOAT64; def : '1e-10'; doc : 'Beam width applied to phone loop transitions for lookahead'),
  (name : '-pl_pip'; typ : ARG_FLOAT32; def : '1.0'; doc : 'Phone insertion penalty for phone loop'),
  (name : '-pl_weight'; typ : ARG_FLOAT64; def : '3.0'; doc : 'Weight for phoneme lookahead penalties'),
  //search
  (name : '-compallsen'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Compute all senone scores in every frame (can be faster when there are many senones)'),
  (name : '-fwdtree'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Run forward lexicon-tree search (1st pass)'),
  (name : '-fwdflat'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Run forward flat-lexicon search over word lattice (2nd pass)'),
  (name : '-bestpath'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Run bestpath (Dijkstra) search over word lattice (3rd pass)'),
  (name : '-backtrace'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Print results and backtraces to log file.'),
  (name : '-latsize'; typ : ARG_INT32; def : '5000'; doc : 'Initial backpointer table size'),
  (name : '-maxwpf'; typ : ARG_INT32; def : '-1'; doc : 'Maximum number of distinct word exits at each frame (or -1 for no pruning)'),
  (name : '-maxhmmpf'; typ : ARG_INT32; def : '30000'; doc : 'Maximum number of active HMMs to maintain at each frame (or -1 for no pruning)'),
  (name : '-min_endfr'; typ : ARG_INT32; def : '0'; doc : 'Nodes ignored in lattice construction if they persist for fewer than N frames'),
  (name : '-fwdflatefwid'; typ : ARG_INT32; def : '4'; doc : 'Minimum number of end frames for a word to be searched in fwdflat search'),
  (name : '-fwdflatsfwin'; typ : ARG_INT32; def : '25'; doc : 'Window of frames in lattice to search for successor words in fwdflat search'),
  // DICT
  (name : '-dict'; typ : REQARG_STRING; def : nil; doc : 'Main pronunciation dictionary (lexicon) input file'),
  (name : '-fdict'; typ : ARG_STRING; def : nil; doc : 'Noise word pronunciation dictionary input file'),
  (name : '-dictcase'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Dictionary is case sensitive (NOTE: case insensitivity applies to ASCII characters only)'),
  // NGRAM
  (name : '-allphone'; typ : ARG_STRING; def : nil; doc : 'Perform phoneme decoding with phonetic lm'),
  (name : '-allphone_ci'; typ : ARG_BOOLEAN; def : 'no'; doc : 'Perform phoneme decoding with phonetic lm and context-independent units only'),
  (name : '-lm'; typ : ARG_STRING; def : nil; doc : 'Word trigram language model input file'),
  (name : '-lmctl'; typ : ARG_STRING; def : nil; doc : 'Specify a set of language model'),
  (name : '-lmname'; typ : ARG_STRING; def : nil; doc : 'Which language model in -lmctl to use by default'),
  (name : '-lw'; typ : ARG_FLOAT32; def : '6.5'; doc : 'Language model probability weight'),
  (name : '-fwdflatlw'; typ : ARG_FLOAT32; def : '8.5'; doc : 'Language model probability weight for flat lexicon (2nd pass) decoding'),
  (name : '-bestpathlw'; typ : ARG_FLOAT32; def : '9.5'; doc : 'Language model probability weight for bestpath search'),
  (name : '-ascale'; typ : ARG_FLOAT32; def : '20.0'; doc : 'Inverse of acoustic model scale for confidence score calculation'),
  (name : '-wip'; typ : ARG_FLOAT32; def : '0.65'; doc : 'Word insertion penalty'),
  (name : '-nwpen'; typ : ARG_FLOAT32; def : '1.0'; doc : 'New word transition penalty'),
  (name : '-pip'; typ : ARG_FLOAT32; def : '1.0'; doc : 'Phone insertion penalty'),
  (name : '-uw'; typ : ARG_FLOAT32; def : '1.0'; doc : 'Unigram weight'),
  (name : '-silprob'; typ : ARG_FLOAT32; def : '0.005'; doc : 'Silence word transition probability'),
  (name : '-fillprob'; typ : ARG_FLOAT32; def : '1e-8'; doc : 'Filler word transition probability'),
  // FSG
  (name : '-fsg'; typ : ARG_STRING; def : nil; doc : 'Sphinx format finite state grammar file'),
  (name : '-jsgf'; typ : ARG_STRING; def : nil; doc : 'JSGF grammar file'),
  (name : '-toprule'; typ : ARG_STRING; def : nil; doc : 'Start rule for JSGF (first public rule is default)'),
  (name : '-fsgusealtpron'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Add alternate pronunciations to FSG'),
  (name : '-fsgusefiller'; typ : ARG_BOOLEAN; def : 'yes'; doc : 'Insert filler words at each state'),
  // KWS
  (name : '-keyphrase'; typ : ARG_STRING; def : nil; doc : 'Keyphrase to spot'),
  (name : '-kws'; typ : ARG_STRING; def : nil; doc : 'A file with keyphrases to spot, one per line'),
  (name : '-kws_plp'; typ : ARG_FLOAT64; def : '1e-1'; doc : 'Phone loop probability for keyphrase spotting'),
  (name : '-kws_delay'; typ : ARG_INT32; def : '10'; doc : 'Delay to wait for best detection score'),
  (name : '-kws_threshold'; typ : ARG_FLOAT64; def : '1'; doc : 'Threshold for p(hyp)/p(alternatives) ratio'),
  // DEBUG
  (name : '-logfn'; typ : ARG_STRING; def : '/dev/null'; doc : 'Log file destination'),
  (name : '-debug'; typ : ARG_INT32; def : nil; doc : 'Verbosity level for debugging messages'),
  (name : '-mfclogdir'; typ : ARG_STRING; def : nil; doc : 'Directory to log feature files to'),
  (name : '-rawlogdir'; typ : ARG_STRING; def : nil; doc : 'Directory to log raw audio files to'),
  (name : '-senlogdir'; typ : ARG_STRING; def : nil; doc : 'Directory to log senone score files to'),
  // the end
  (name : nil; typ : 0; def : nil; doc : nil));


// functions

function cmd_ln_parse_r(cmdLn: PpsConfig; const defaults: PpsArg; argc: int32; argv: pointer; strict: WordBool): PpsConfig; cdecl; external SB_lib_name name 'cmd_ln_parse_r';
function cmd_ln_str_r(config: PpsConfig; name: PChar): PChar; cdecl; external SB_lib_name name 'cmd_ln_str_r';
function cmd_ln_free_r(cmdLn: PpsConfig): integer; cdecl; external SB_lib_name name 'cmd_ln_free_r';

procedure ps_default_search_args(const cmdLn: PpsConfig); cdecl; external PS_lib_name name 'ps_default_search_args';
function ps_args: PpsArg; cdecl; external PS_lib_name name 'ps_args';
function ps_init(const cmdLn: PpsConfig): PpsDecoder; cdecl; external PS_lib_name name 'ps_init';
function ps_free(decoder: PpsDecoder): integer; cdecl; external PS_lib_name name 'ps_free';
function ps_start_utt(decoder: PpsDecoder): integer; cdecl; external PS_lib_name name 'ps_start_utt';
function ps_process_raw(decoder: PpsDecoder; const buf: PpsBuffer; size: integer; no_search, full_utt: integer): integer; cdecl; external PS_lib_name name 'ps_process_raw';
function ps_get_in_speech(decoder: PpsDecoder): byte; cdecl; external PS_lib_name name 'ps_get_in_speech';
function ps_end_utt(decoder: PpsDecoder): integer; cdecl; external PS_lib_name name 'ps_end_utt';
function ps_get_hyp(decoder: PpsDecoder; best_score: pInt32): PChar; cdecl; external PS_lib_name name 'ps_get_hyp';

implementation

end.

